@@ -28,6 +28,25 @@
 #include <string.h>
 #include <glib/gi18n.h>
 #include "gedit-spell-utils.h"
+#include "gtktextregion.h"
+
+/**
+ * SECTION:automatic-spell-checker
+ * @Short_description: Inline spell checker for GtkTextView
+ * @Title: GeditAutomaticSpellChecker
+ * @See_also: #GeditSpellChecker
+ *
+ * The #GeditAutomaticSpellChecker is an inline spell checker for the
+ * #GtkTextView widget. Misspelled words are highlighted with a
+ * %PANGO_UNDERLINE_ERROR, usually a red wavy underline. Right-clicking a
+ * misspelled word pops up a context menu of suggested replacements. The context
+ * menu also contains an “Ignore All” item to add the misspelled word to the
+ * session dictionary. And an “Add” item to add the word to the personal
+ * dictionary.
+ *
+ * The spell is checked only on the visible regions of the attached
+ * #GtkTextView's.
+ */
 
 struct _GeditAutomaticSpellChecker
 {
@@ -41,11 +60,10 @@ struct _GeditAutomaticSpellChecker
 
 	GtkTextTag *tag_highlight;
 
-	GtkTextMark *mark_insert_start;
-	GtkTextMark *mark_insert_end;
 	GtkTextMark *mark_click;
 
-	guint deferred_check : 1;
+	GtkTextRegion *scan_region;
+	guint timeout_id;
 };
 
 enum
@@ -55,9 +73,17 @@ enum
 	PROP_SPELL_CHECKER,
 };
 
+#define ENABLE_DEBUG 0
+
 #define AUTOMATIC_SPELL_CHECKER_KEY	"GeditAutomaticSpellCheckerID"
 #define SUGGESTION_KEY			"GeditAutoSuggestionID"
 
+/* Timeout durations in milliseconds. Writing and deleting text should be smooth
+ * and responsive.
+ */
+#define TIMEOUT_DURATION_BUFFER_MODIFIED 400
+#define TIMEOUT_DURATION_DRAWING 20
+
 G_DEFINE_TYPE (GeditAutomaticSpellChecker, gedit_automatic_spell_checker, G_TYPE_OBJECT)
 
 static void
@@ -66,10 +92,29 @@ check_word (GeditAutomaticSpellChecker *spell,
 	    const GtkTextIter          *end)
 {
 	gchar *word;
+	GError *error = NULL;
+	gboolean correctly_spelled;
+
+	if (!gtk_text_iter_starts_word (start) ||
+	    !gtk_text_iter_ends_word (end))
+	{
+		g_warning ("Spell checking: @start and @end must delimit a word");
+		return;
+	}
 
 	word = gtk_text_buffer_get_text (spell->buffer, start, end, FALSE);
 
-	if (!gedit_spell_checker_check_word (spell->spell_checker, word))
+	correctly_spelled = gedit_spell_checker_check_word (spell->spell_checker,
+							    word,
+							    &error);
+
+	if (error != NULL)
+	{
+		g_warning ("Automatic spell checker: %s", error->message);
+		g_error_free (error);
+	}
+
+	if (!correctly_spelled)
 	{
 		gtk_text_buffer_apply_tag (spell->buffer,
 					   spell->tag_highlight,
@@ -112,27 +157,13 @@ adjust_iters_at_word_boundaries (GtkTextIter *start,
 }
 
 static void
-check_range (GeditAutomaticSpellChecker *spell,
-	     const GtkTextIter          *start,
-	     const GtkTextIter          *end,
-	     gboolean                    force_all)
+check_subregion (GeditAutomaticSpellChecker *spell,
+		 const GtkTextIter          *start,
+		 const GtkTextIter          *end)
 {
-	GtkTextIter cursor;
-	GtkTextIter precursor;
 	GtkTextIter start_adjusted;
 	GtkTextIter end_adjusted;
 	GtkTextIter word_start;
-  	gboolean highlight;
-
-	gtk_text_buffer_get_iter_at_mark (spell->buffer,
-					  &cursor,
-					  gtk_text_buffer_get_insert (spell->buffer));
-
-	precursor = cursor;
-	gtk_text_iter_backward_char (&precursor);
-
-	highlight = (gtk_text_iter_has_tag (&cursor, spell->tag_highlight) ||
-		     gtk_text_iter_has_tag (&precursor, spell->tag_highlight));
 
 	start_adjusted = *start;
 	end_adjusted = *end;
@@ -150,33 +181,14 @@ check_range (GeditAutomaticSpellChecker *spell,
 	{
 		GtkTextIter word_end;
 		GtkTextIter next_word_start;
-		gboolean in_word;
 
 		word_end = word_start;
 		gtk_text_iter_forward_word_end (&word_end);
 
-		in_word = (gtk_text_iter_compare (&word_start, &cursor) < 0 &&
-			   gtk_text_iter_compare (&cursor, &word_end) <= 0);
-
-		if (in_word && !force_all)
-		{
-			/* This word is being actively edited,
-			 * only check if it's already highligted,
-			 * otherwise defer this check until later.
-			 */
-			if (highlight)
-			{
-				check_word (spell, &word_start, &word_end);
-			}
-			else
-			{
-				spell->deferred_check = TRUE;
-			}
-		}
-		else
+		if (gtk_text_iter_starts_word (&word_start) &&
+		    gtk_text_iter_ends_word (&word_end))
 		{
 			check_word (spell, &word_start, &word_end);
-			spell->deferred_check = FALSE;
 		}
 
 		next_word_start = word_end;
@@ -196,20 +208,223 @@ check_range (GeditAutomaticSpellChecker *spell,
 }
 
 static void
-check_deferred_range (GeditAutomaticSpellChecker *spell,
-		      gboolean                    force_all)
+check_region (GeditAutomaticSpellChecker *spell,
+	      GtkTextRegion              *region)
 {
-	GtkTextIter start, end;
+	GtkTextRegionIterator region_iter;
 
-	gtk_text_buffer_get_iter_at_mark (spell->buffer,
-					  &start,
-					  spell->mark_insert_start);
+	if (region == NULL)
+	{
+		return;
+	}
 
-	gtk_text_buffer_get_iter_at_mark (spell->buffer,
-					  &end,
-					  spell->mark_insert_end);
+	gtk_text_region_get_iterator (region, &region_iter, 0);
 
-	check_range (spell, &start, &end, force_all);
+	while (!gtk_text_region_iterator_is_end (&region_iter))
+	{
+		GtkTextIter start;
+		GtkTextIter end;
+
+		if (!gtk_text_region_iterator_get_subregion (&region_iter, &start, &end))
+		{
+			return;
+		}
+
+		check_subregion (spell, &start, &end);
+
+		gtk_text_region_iterator_next (&region_iter);
+	}
+}
+
+static void
+get_visible_region (GtkTextView *view,
+		    GtkTextIter *start,
+		    GtkTextIter *end)
+{
+	GdkRectangle visible_rect;
+
+	gtk_text_view_get_visible_rect (view, &visible_rect);
+
+	gtk_text_view_get_line_at_y (view,
+				     start,
+				     visible_rect.y,
+				     NULL);
+
+	gtk_text_view_get_line_at_y (view,
+				     end,
+				     visible_rect.y + visible_rect.height,
+				     NULL);
+
+	gtk_text_iter_backward_line (start);
+	gtk_text_iter_forward_line (end);
+}
+
+/* A TextRegion can contain empty subregions. So checking the number of
+ * subregions is not sufficient.
+ * When calling gtk_text_region_add() with equal iters, the subregion is not
+ * added. But when a subregion becomes empty, due to text deletion, the
+ * subregion is not removed from the TextRegion.
+ */
+static gboolean
+is_text_region_empty (GtkTextRegion *region)
+{
+	GtkTextRegionIterator region_iter;
+
+	if (region == NULL)
+	{
+		return TRUE;
+	}
+
+	gtk_text_region_get_iterator (region, &region_iter, 0);
+
+	while (!gtk_text_region_iterator_is_end (&region_iter))
+	{
+		GtkTextIter region_start;
+		GtkTextIter region_end;
+
+		if (!gtk_text_region_iterator_get_subregion (&region_iter,
+							     &region_start,
+							     &region_end))
+		{
+			return TRUE;
+		}
+
+		if (!gtk_text_iter_equal (&region_start, &region_end))
+		{
+			return FALSE;
+		}
+
+		gtk_text_region_iterator_next (&region_iter);
+	}
+
+	return TRUE;
+}
+
+static void
+check_visible_region_in_view (GeditAutomaticSpellChecker *spell,
+			      GtkTextView                *view)
+{
+	GtkTextIter visible_start;
+	GtkTextIter visible_end;
+	GtkTextRegion *intersect;
+
+	if (spell->scan_region == NULL)
+	{
+		return;
+	}
+
+	get_visible_region (view, &visible_start, &visible_end);
+
+	intersect = gtk_text_region_intersect (spell->scan_region,
+					       &visible_start,
+					       &visible_end);
+
+	if (intersect == NULL)
+	{
+		return;
+	}
+
+	check_region (spell, intersect);
+	gtk_text_region_destroy (intersect);
+
+	gtk_text_region_subtract (spell->scan_region,
+				  &visible_start,
+				  &visible_end);
+
+	if (is_text_region_empty (spell->scan_region))
+	{
+		gtk_text_region_destroy (spell->scan_region);
+		spell->scan_region = NULL;
+	}
+}
+
+static void
+check_visible_region (GeditAutomaticSpellChecker *spell)
+{
+	GSList *l;
+#if ENABLE_DEBUG
+	GTimer *timer;
+#endif
+
+	if (spell->scan_region == NULL)
+	{
+		return;
+	}
+
+#if ENABLE_DEBUG
+	timer = g_timer_new ();
+#endif
+
+	for (l = spell->views; l != NULL; l = l->next)
+	{
+		GtkTextView *view = l->data;
+		check_visible_region_in_view (spell, view);
+	}
+
+#if ENABLE_DEBUG
+	g_print ("%s() executed in %lf seconds\n",
+		 G_STRFUNC,
+		 g_timer_elapsed (timer, NULL));
+
+	g_timer_destroy (timer);
+#endif
+}
+
+static gboolean
+timeout_cb (GeditAutomaticSpellChecker *spell)
+{
+	check_visible_region (spell);
+
+	spell->timeout_id = 0;
+	return G_SOURCE_REMOVE;
+}
+
+static void
+install_timeout (GeditAutomaticSpellChecker *spell,
+		 guint                       duration)
+{
+	if (spell->timeout_id != 0)
+	{
+		g_source_remove (spell->timeout_id);
+	}
+
+	spell->timeout_id = g_timeout_add (duration,
+					   (GSourceFunc) timeout_cb,
+					   spell);
+}
+
+static void
+add_subregion_to_scan (GeditAutomaticSpellChecker *spell,
+		       const GtkTextIter          *start,
+		       const GtkTextIter          *end)
+{
+	GtkTextIter start_adjusted;
+	GtkTextIter end_adjusted;
+
+	if (spell->scan_region == NULL)
+	{
+		spell->scan_region = gtk_text_region_new (spell->buffer);
+	}
+
+	start_adjusted = *start;
+	end_adjusted = *end;
+	adjust_iters_at_word_boundaries (&start_adjusted, &end_adjusted);
+
+	gtk_text_region_add (spell->scan_region,
+			     &start_adjusted,
+			     &end_adjusted);
+}
+
+static void
+recheck_all (GeditAutomaticSpellChecker *spell)
+{
+	GtkTextIter start;
+	GtkTextIter end;
+
+	gtk_text_buffer_get_bounds (spell->buffer, &start, &end);
+
+	add_subregion_to_scan (spell, &start, &end);
+	check_visible_region (spell);
 }
 
 static void
@@ -223,13 +438,10 @@ insert_text_after_cb (GtkTextBuffer              *buffer,
 	GtkTextIter end;
 
 	start = end = *location;
-
 	gtk_text_iter_backward_chars (&start, g_utf8_strlen (text, length));
 
-	gtk_text_buffer_move_mark (buffer, spell->mark_insert_start, &start);
-	gtk_text_buffer_move_mark (buffer, spell->mark_insert_end, &end);
-
-	check_range (spell, &start, &end, FALSE);
+	add_subregion_to_scan (spell, &start, &end);
+	install_timeout (spell, TIMEOUT_DURATION_BUFFER_MODIFIED);
 }
 
 static void
@@ -238,20 +450,8 @@ delete_range_after_cb (GtkTextBuffer              *buffer,
 		       GtkTextIter                *end,
 		       GeditAutomaticSpellChecker *spell)
 {
-	check_range (spell, start, end, FALSE);
-}
-
-static void
-mark_set_cb (GtkTextBuffer              *buffer,
-	     GtkTextIter                *iter,
-	     GtkTextMark                *mark,
-	     GeditAutomaticSpellChecker *spell)
-{
-	/* if the cursor has moved and there is a deferred check so handle it now */
-	if ((mark == gtk_text_buffer_get_insert (buffer)) && spell->deferred_check)
-	{
-		check_deferred_range (spell, FALSE);
-	}
+	add_subregion_to_scan (spell, start, end);
+	install_timeout (spell, TIMEOUT_DURATION_BUFFER_MODIFIED);
 }
 
 static gboolean
@@ -490,6 +690,16 @@ populate_popup_cb (GtkTextView                *view,
 	gtk_widget_show_all (menu_item);
 }
 
+static gboolean
+draw_cb (GtkWidget                  *text_view,
+	 cairo_t                    *cr,
+	 GeditAutomaticSpellChecker *spell)
+{
+	install_timeout (spell, TIMEOUT_DURATION_DRAWING);
+
+	return GDK_EVENT_PROPAGATE;
+}
+
 static void
 remove_tag_to_word (GeditAutomaticSpellChecker *spell,
 		    const gchar                *word)
@@ -542,15 +752,15 @@ static void
 clear_session_cb (GeditSpellChecker          *checker,
 		  GeditAutomaticSpellChecker *spell)
 {
-	gedit_automatic_spell_checker_recheck_all (spell);
+	recheck_all (spell);
 }
 
 static void
-set_language_cb (GeditSpellChecker               *checker,
-		 const GeditSpellCheckerLanguage *lang,
-		 GeditAutomaticSpellChecker      *spell)
+language_notify_cb (GeditSpellChecker          *checker,
+		    GParamSpec                 *pspec,
+		    GeditAutomaticSpellChecker *spell)
 {
-	gedit_automatic_spell_checker_recheck_all (spell);
+	recheck_all (spell);
 }
 
 /* When the user right-clicks on a word, they want to check that word.
@@ -569,11 +779,6 @@ button_press_event_cb (GtkTextView                *view,
 		gint x;
 		gint y;
 
-		if (spell->deferred_check)
-		{
-			check_deferred_range (spell, TRUE);
-		}
-
 		gtk_text_view_window_to_buffer_coords (view,
 						       GTK_TEXT_WINDOW_TEXT,
 						       event->x, event->y,
@@ -596,11 +801,6 @@ popup_menu_cb (GtkTextView                *view,
 {
 	GtkTextIter iter;
 
-	if (spell->deferred_check)
-	{
-		check_deferred_range (spell, TRUE);
-	}
-
 	gtk_text_buffer_get_iter_at_mark (spell->buffer, &iter,
 					  gtk_text_buffer_get_insert (spell->buffer));
 	gtk_text_buffer_move_mark (spell->buffer, spell->mark_click, &iter);
@@ -652,7 +852,8 @@ highlight_updated_cb (GtkSourceBuffer            *buffer,
 		      GtkTextIter                *end,
 		      GeditAutomaticSpellChecker *spell)
 {
-	check_range (spell, start, end, FALSE);
+	add_subregion_to_scan (spell, start, end);
+	install_timeout (spell, TIMEOUT_DURATION_BUFFER_MODIFIED);
 }
 
 static void
@@ -661,13 +862,10 @@ set_buffer (GeditAutomaticSpellChecker *spell,
 {
 	GtkTextTagTable *tag_table;
 	GtkTextIter start;
-	GtkTextIter end;
 
 	g_return_if_fail (GTK_SOURCE_IS_BUFFER (buffer));
 	g_return_if_fail (spell->buffer == NULL);
 	g_return_if_fail (spell->tag_highlight == NULL);
-	g_return_if_fail (spell->mark_insert_start == NULL);
-	g_return_if_fail (spell->mark_insert_end == NULL);
 	g_return_if_fail (spell->mark_click == NULL);
 
 	spell->buffer = g_object_ref (buffer);
@@ -688,12 +886,6 @@ set_buffer (GeditAutomaticSpellChecker *spell,
 				 spell,
 				 G_CONNECT_AFTER);
 
-	g_signal_connect_object (buffer,
-				 "mark-set",
-				 G_CALLBACK (mark_set_cb),
-				 spell,
-				 0);
-
 	g_signal_connect_object (buffer,
 				 "highlight-updated", /* GtkSourceBuffer signal */
 				 G_CALLBACK (highlight_updated_cb),
@@ -725,14 +917,15 @@ set_buffer (GeditAutomaticSpellChecker *spell,
 				 spell,
 				 0);
 
-	/* We create the mark here, but we don't use it until text is
-	 * inserted, so we don't really care where iter points.
+	/* For now we don't care where the mark points. The start looks like a
+	 * good place to begin with.
 	 */
-	gtk_text_buffer_get_bounds (spell->buffer, &start, &end);
-
-	spell->mark_insert_start = gtk_text_buffer_create_mark (spell->buffer, NULL, &start, TRUE);
-	spell->mark_insert_end = gtk_text_buffer_create_mark (spell->buffer, NULL, &start, TRUE);
+	gtk_text_buffer_get_start_iter (spell->buffer, &start);
 	spell->mark_click = gtk_text_buffer_create_mark (spell->buffer, NULL, &start, TRUE);
+
+	recheck_all (spell);
+
+	g_object_notify (G_OBJECT (spell), "buffer");
 }
 
 static void
@@ -763,10 +956,12 @@ set_spell_checker (GeditAutomaticSpellChecker *spell,
 				 0);
 
 	g_signal_connect_object (spell->spell_checker,
-				 "set_language",
-				 G_CALLBACK (set_language_cb),
+				 "notify::language",
+				 G_CALLBACK (language_notify_cb),
 				 spell,
 				 0);
+
+	g_object_notify (G_OBJECT (spell), "spell-checker");
 }
 
 static void
@@ -833,16 +1028,6 @@ gedit_automatic_spell_checker_dispose (GObject *object)
 			gtk_text_tag_table_remove (table, spell->tag_highlight);
 		}
 
-		if (spell->mark_insert_start != NULL)
-		{
-			gtk_text_buffer_delete_mark (spell->buffer, spell->mark_insert_start);
-			spell->mark_insert_start = NULL;
-		}
-		if (spell->mark_insert_end != NULL)
-		{
-			gtk_text_buffer_delete_mark (spell->buffer, spell->mark_insert_end);
-			spell->mark_insert_end = NULL;
-		}
 		if (spell->mark_click != NULL)
 		{
 			gtk_text_buffer_delete_mark (spell->buffer, spell->mark_click);
@@ -861,10 +1046,20 @@ gedit_automatic_spell_checker_dispose (GObject *object)
 	g_slist_free_full (spell->views, g_object_unref);
 	spell->views = NULL;
 
-	spell->mark_insert_start = NULL;
-	spell->mark_insert_end = NULL;
 	spell->mark_click = NULL;
 
+	if (spell->scan_region != NULL)
+	{
+		gtk_text_region_destroy (spell->scan_region);
+		spell->scan_region = NULL;
+	}
+
+	if (spell->timeout_id != 0)
+	{
+		g_source_remove (spell->timeout_id);
+		spell->timeout_id = 0;
+	}
+
 	G_OBJECT_CLASS (gedit_automatic_spell_checker_parent_class)->dispose (object);
 }
 
@@ -877,6 +1072,13 @@ gedit_automatic_spell_checker_class_init (GeditAutomaticSpellCheckerClass *klass
 	object_class->set_property = gedit_automatic_spell_checker_set_property;
 	object_class->dispose = gedit_automatic_spell_checker_dispose;
 
+	/**
+	 * GeditAutomaticSpellChecker:buffer:
+	 *
+	 * The #GtkSourceBuffer. If a same buffer is used for several views, the
+	 * misspelled words are visible in all views, because #GtkTextTag's are
+	 * added to the buffer.
+	 */
 	g_object_class_install_property (object_class,
 					 PROP_BUFFER,
 					 g_param_spec_object ("buffer",
@@ -887,6 +1089,13 @@ gedit_automatic_spell_checker_class_init (GeditAutomaticSpellCheckerClass *klass
 							      G_PARAM_CONSTRUCT_ONLY |
 							      G_PARAM_STATIC_STRINGS));
 
+	/**
+	 * GeditAutomaticSpellChecker:spell-checker:
+	 *
+	 * The #GeditSpellChecker to use. It cannot be changed afterwards. If
+	 * you want to change the language, you need to create another
+	 * #GeditAutomaticSpellChecker.
+	 */
 	g_object_class_install_property (object_class,
 					 PROP_SPELL_CHECKER,
 					 g_param_spec_object ("spell-checker",
@@ -901,9 +1110,15 @@ gedit_automatic_spell_checker_class_init (GeditAutomaticSpellCheckerClass *klass
 static void
 gedit_automatic_spell_checker_init (GeditAutomaticSpellChecker *spell)
 {
-	spell->deferred_check = FALSE;
 }
 
+/**
+ * gedit_automatic_spell_checker_new:
+ * @buffer: a #GtkSourceBuffer.
+ * @checker: a #GeditSpellChecker.
+ *
+ * Returns: a new #GeditAutomaticSpellChecker object.
+ */
 GeditAutomaticSpellChecker *
 gedit_automatic_spell_checker_new (GtkSourceBuffer   *buffer,
 				   GeditSpellChecker *checker)
@@ -927,6 +1142,14 @@ gedit_automatic_spell_checker_new (GtkSourceBuffer   *buffer,
 			     NULL);
 }
 
+/**
+ * gedit_automatic_spell_checker_attach_view:
+ * @spell: a #GeditAutomaticSpellChecker.
+ * @view: a #GtkTextView.
+ *
+ * Enables the inline spell checker for @view. @view must have the same buffer as
+ * the #GeditAutomaticSpellChecker:buffer property.
+ */
 void
 gedit_automatic_spell_checker_attach_view (GeditAutomaticSpellChecker *spell,
 					   GtkTextView                *view)
@@ -954,10 +1177,25 @@ gedit_automatic_spell_checker_attach_view (GeditAutomaticSpellChecker *spell,
 				 spell,
 				 0);
 
+	g_signal_connect_object (view,
+				 "draw",
+				 G_CALLBACK (draw_cb),
+				 spell,
+				 0);
+
 	spell->views = g_slist_prepend (spell->views, view);
 	g_object_ref (view);
+
+	check_visible_region_in_view (spell, view);
 }
 
+/**
+ * gedit_automatic_spell_checker_detach_view:
+ * @spell: a #GeditAutomaticSpellChecker.
+ * @view: a #GtkTextView.
+ *
+ * Disables the inline spell checker for @view.
+ */
 void
 gedit_automatic_spell_checker_detach_view (GeditAutomaticSpellChecker *spell,
 					   GtkTextView                *view)
@@ -973,17 +1211,4 @@ gedit_automatic_spell_checker_detach_view (GeditAutomaticSpellChecker *spell,
 	g_object_unref (view);
 }
 
-void
-gedit_automatic_spell_checker_recheck_all (GeditAutomaticSpellChecker *spell)
-{
-	GtkTextIter start;
-	GtkTextIter end;
-
-	g_return_if_fail (GEDIT_IS_AUTOMATIC_SPELL_CHECKER (spell));
-
-	gtk_text_buffer_get_bounds (spell->buffer, &start, &end);
-
-	check_range (spell, &start, &end, TRUE);
-}
-
 /* ex:set ts=8 noet: */
